
' http://www.freebasic-portal.de/code-beispiele/kleine-helferlein/bipipe-bi-fuer-win-linux-299.html

#if defined(__FB_WIN32__)
#include once "windows.bi"
#elseif defined(__FB_LINUX__)
#include once "crt/linux/unistd.bi"
declare function ioctl alias "ioctl" (fd as integer, request as ulong, ...) as integer
#define FIONREAD &h541B
#define SW_NORMAL 0
#endif

type BiPipe
  private:
#if defined(__FB_WIN32__)
  hProcessHandle as HANDLE
  hWritePipe as HANDLE
  hReadPipe as HANDLE
#elseif defined(__FB_LINUX__)
  pipeStdin as long
  pipeStdout as long
#endif
  public:
#if defined(__FB_WIN32__)
  declare constructor (prgName as string, showmode as short = SW_NORMAL)
#elseif defined(__FB_LINUX__)
  declare constructor (prgName as string)
#endif
  declare destructor ()
  declare function write (text as string) as integer
  declare function read (timeout as uinteger = 100) as string
#if defined(__FB_WIN32__)
  declare function readLine (separator as string = "a" & chr(13, 10), timeout as uinteger = 100) as string
#endif
end type

#if defined(__FB_WIN32__)
constructor BiPipe (prgName as string, showmode as short = SW_NORMAL)
  dim as STARTUPINFO si
  dim as PROCESS_INFORMATION pi
  dim as SECURITY_ATTRIBUTES sa
  dim as HANDLE hReadPipe, hWritePipe, hReadChildPipe, hWriteChildPipe
  'set security attributes
  sa.nLength = sizeof(SECURITY_ATTRIBUTES)
  sa.lpSecurityDescriptor = NULL 'use default descriptor
  sa.bInheritHandle = TRUE
  'create one pipe for each direction
  CreatePipe(@hReadChildPipe,@hWritePipe,@sa,0) 'parent to child
  CreatePipe(@hReadPipe,@hWriteChildPipe,@sa,0) 'child to parent
  GetStartupInfo(@si)
  si.dwFlags = STARTF_USESTDHANDLES or STARTF_USESHOWWINDOW
  si.wShowWindow = showmode 'appearance of child process window
  si.hStdOutput  = hWriteChildPipe
  si.hStdError   = hWriteChildPipe
  si.hStdInput   = hReadChildPipe
  CreateProcess(0,PrgName,0,0,TRUE,CREATE_NEW_CONSOLE,0,0,@si,@pi)
  CloseHandle(hWriteChildPipe)
  CloseHandle(hReadChildPipe)
  this.hProcessHandle = pi.hProcess 'handle to child process
  this.hWritePipe = hWritePipe
  this.hReadPipe = hReadPipe
end constructor

destructor BiPipe ()
    TerminateProcess(hProcessHandle, 0)
    CloseHandle(hWritePipe)
    CloseHandle(hReadPipe)
end destructor

function BiPipe.write (text as string) as integer
  dim bytesWritten as integer
  WriteFile(hWritePipe, strptr(text), len(text), @bytesWritten, 0)
  return bytesWritten
end function

function BiPipe.read (timeout as uinteger = 100) as string
' Returns the whole pipe content until the pipe is empty or timeout occurs.
' Timeout default is 100ms to prevent a deadlock.
  dim as integer iNumberOfBytesRead, iTotalBytesAvail, iBytesLeftThisMessage
  dim as string buffer, retText
  dim as double tout = timer + cast(double,timeout) / 1000
  do
    PeekNamedPipe(hReadPipe,0,0,0,@iTotalBytesAvail,0)
    if iTotalBytesAvail then
      buffer = string(iTotalBytesAvail,chr(0))
      ReadFile(hReadPipe,strptr(buffer),len(buffer),@iNumberOfBytesRead,0)
      retText &= buffer
    elseif len(retText) then
      exit do
    endif
  loop until timer > tout
  return retText
end function

function BiPipe.readLine (separator as string = "a" & chr(13,10), timeout as uinteger = 100) as string
' Returns the pipe content till the first separator if any, or otherwise the whole pipe
' content on timeout. Timeout default is 100ms to prevent a deadlock.
  dim as integer iNumberOfBytesRead, iTotalBytesAvail, iBytesLeftThisMessage, endPtr
  dim as string buffer, retText, mode
  dim as double tout = timer + cast(double,timeout) / 1000
  mode = lcase(left(separator, 1))
  separator = mid(separator, 2)
  do
      PeekNamedPipe(hReadPipe,0,0,0,@iTotalBytesAvail,0)
      if iTotalBytesAvail then
          buffer = string(iTotalBytesAvail,chr(0))
          PeekNamedPipe(hReadPipe,strptr(buffer),len(buffer),@iNumberOfBytesRead, _
          @iTotalBytesAvail,@iBytesLeftThisMessage) 'copy pipe content to buffer
          select case mode
              case "a" 'any
                  endPtr = instr(buffer, any separator) 'look for line end sign
              case "e" 'exact
                  endPtr = instr(buffer, separator) 'look for line end sign
          end select
          if endPtr then 'return pipe content till line end
              select case mode
                  case "a"
                      do while (instr(separator,chr(buffer[endPtr - 1]))) and (endPtr < len(buffer))
                          endPtr += 1
                      loop
                      endPtr -= 1
                  case "e"
                      endPtr += len(separator)
              end select
              retText = left(buffer,endPtr)
              ReadFile(hReadPipe,strptr(buffer),endPtr,@iNumberOfBytesRead,0) 'remove read bytes from pipe
              select case mode
                  case "a"
                      return rtrim(retText,any separator) 'remove line end sign from returned string
                  case "e"
                      return left(retText,len(retText) - len(separator))
              end select
          endif
      endif
  loop until timer > tout
  if iTotalBytesAvail then 'return all pipe content
      buffer = string(iTotalBytesAvail,chr(0))
      ReadFile(hReadPipe,strptr(buffer),len(buffer),@iNumberOfBytesRead,0)
      return buffer
  endif
  return ""
end function
#elseif defined(__FB_LINUX__)
constructor BiPipe (prgName as string)
    dim pipeStdin(0 to 1) as long
    dim pipeStdout(0 to 1) as long

    pipe_(@pipeStdin(0))
    pipe_(@pipeStdout(0))

    if fork() = 0 then
        close_(pipeStdin(1))
        close_(pipeStdout(0))

        dup2(pipeStdin(0), 0)
        dup2(pipeStdout(1), 1)

        execl(strptr("/bin/sh"), strptr("sh"), strptr("-c"), strptr(prgName), cast(ubyte ptr, 0))
        _exit(1)
    end if

    this.pipeStdin = pipeStdin(1)
    this.pipeStdout = pipeStdout(0)

    close_(pipeStdin(0))
    close_(pipeStdout(1))
end constructor

destructor BiPipe ()
    close_(pipeStdin)
    close_(pipeStdout)
end destructor

function BiPipe.write (text as string) as integer
    return write_(pipeStdin, strptr(text), len(text))
end function

function BiPipe.read (timeout as uinteger = 100) as string
    'returns the whole pipe content until the pipe is empty or timeout occurs.
    ' timeout default is 100ms to prevent a deadlock
    dim as integer iNumberOfBytesRead, iTotalBytesAvail, iBytesLeftThisMessage
    dim as string buffer, retText
    dim as double tout = timer + cast(double,timeout) / 1000
    do
        ioctl(pipeStdout, FIONREAD, @iTotalBytesAvail)
        if iTotalBytesAvail then
            buffer = string(iTotalBytesAvail,chr(0))
            read_(pipeStdout, strptr(buffer), len(buffer))
            retText &= buffer
        elseif len(retText) then
            exit do
        end if
    loop until timer > tout
    return retText
end function
#endif

function bipOpen(PrgName as string, showmode as short = SW_NORMAL) as BiPipe ptr
#if defined(__FB_WIN32__)
  return new BiPipe(PrgName, showmode)
#elseif defined(__FB_LINUX__)
  return new BiPipe(PrgName)
#endif
end function

sub bipClose(byref handles as BiPipe ptr)
  delete handles
  handles = 0
end sub

function bipWrite(handles as BiPipe ptr, text as string) as integer
  return handles->write(text)
end function

function bipRead(handles as BiPipe ptr, timeout as uinteger = 100) as string
  return handles->read(timeout)
end function

function bipReadLine(handles as BiPipe ptr, separator as string = "a" & chr(13, 10), timeout as uinteger = 100) as string
#if defined(__FB_WIN32__)
  return handles->readLine(separator, timeout)
#elseif defined(__FB_LINUX__)
#print __function__ Function not implemented
  return ""
#endif
end function
