' ======================================================================
' Eschecs FreeBASIC
' ======================================================================

#include once "vbcompat.bi"

type TListener
  dim handle as any ptr = 0
  dim sync as any ptr = 0
  dim quit as boolean = false
  dim procedure as sub (aOutput as const string) = 0
  dim doReadOutput as boolean = false
end type

declare sub ProcedureThread(byval param as any ptr)
declare sub OnOutput(aOutput as const string)

dim shared listener as TListener ptr
listener = new TListener
listener->sync = mutexcreate
listener->procedure = @OnOutput
listener->handle = threadcreate(@ProcedureThread, listener)

#ifdef DEBUG
dim shared console as integer
console = freefile
open cons for output as console
#define LINE_ID __FUNCTION__ & " (" & __LINE__ & ")"
#define DebugLn(_X_) print #console, LINE_ID & " " & (_X_)
#else
#define DebugLn(_X_)
#endif

#include once "fbjson/fbjson.bi"
#include once "stringlist.bi"
#include once "sgui/sgui/sgui.bas"
#include once "sgui/sgui/menu.bas"
#include once "sgui/special/eschecs/clickmapgadget.bas"
#include once "command.bi"
#include once "connect.bi"
#include once "parser.bi"
#include once "chess/chess.bi"
#include once "colors.bi"
#include once "graphics.bi"
#include once "language.bi"
#include once "validator.bi"
#include once "version.bi"
#define SOUND_EFFECTS
#include once "sound.bi"
#include once "inifile.bi"

#define LOG_NAME         exepath & "/eschecs.log"
#define INI_NAME         exepath & "/eschecs.ini"
#define SOUND_ILLEGAL    exepath & "/resources/sounds/illegal.wav"
#define OPENING_DATABASE exepath & "/resources/eco/eco.json"
#define ENGINES_CONFIG   exepath & "/engines/" & lcase(OS) & "/engines.json"

using sGUI
using inifobj
using strlist

declare sub DisplayArbitratorMessage(byval as string)
declare function IsPromotionMove(byref as chess.tChessGame, byval as string) as boolean
declare function IsCastlingMove(byref as chess.tChessGame, byval as string) as boolean
declare function IsEnPassantMove(byref as chess.tChessGame, byval as string) as boolean
declare function ReplaceFieldName(byval as string) as string
declare function ComparePiecePlacement(byval as string, byval as string) as boolean
declare function ComputerToMove as boolean
declare function LoadFileAsString(byval filename as string) as string
declare sub LoadEnginesData(aFileName as const string)
declare function ConnectEngine(aIndex as const integer) as boolean
declare function FormatEngineOutput(byref aStr as const string) as string
declare sub OnUciOk(aOutput as const string)
declare sub OnBestMove(aOutput as const string)
declare function DetectRepetition as boolean
declare sub SendCommandLog(byref aCommand as const string)
declare sub OnNewPosition
declare sub OnLegalMove(byref aUserMove as const boolean)
declare sub OnTryConnect(byref aSuccess as const boolean)
declare sub DisplayMove
declare function StrToIntDef(byref aStr as const string, byref default as const integer) as integer
declare sub RepaintBackground
declare sub DrawCoord
declare sub LoadSettings(byref aMoveTime as integer, byref aSquareWidth as integer)
declare sub SaveSettings(byref aMoveTime as const integer, byref aSquareWidth as const integer)

type chessfield 
  fieldname as string 
  selected as integer 
  posx as integer 
  posy as integer 
end type

type TEngineInfo
  as string eiCommand, eiName, eiWorkingDirectory
  as boolean eiExists
end type

dim moveTime as integer = 0
LoadSettings(moveTime, squareWidth)

dim commandLine as TCommandLine
dim s as string = commandLine.GetOptionValue("moveTime")
moveTime = StrToIntDef(s, moveTime)
s = commandLine.GetOptionValue("squareWidth")
squareWidth = StrToIntDef(s, squareWidth)
if (squareWidth <> 40) and (squareWidth <> 48) then
  squareWidth = 40
end if

const textHeight = 14
dim as integer w, h
screenres BOARD_WIDTH + squareWidth, 18 + BOARD_WIDTH + 18 + squareWidth, 32
screeninfo w, h
width w \ 8, h \ textHeight
SetLanguage
windowtitle(PROGRAM_NAME & " " & PROGRAM_VERSION)

dim shared as uinteger messageTextColor
dim shared as uinteger messageColor
customColors = true
' sGUI colors
backgroundColor	= colors.lightGray
textColor	= black
gadgetColor	= colors.lemonChiffon'lightYellow'orange
menuColor = colors.lightSteelBlue'lightPink'gainsboro'aliceBlue'cornFlowerBlue'lightGray'darkOrange
gadgetGlowColor	= &hFF00FF
gadgetTextColor	= &hFF00FF
cursorColor = &hFF00FF
menuTextColor = black
menuHiliteColor = colors.lemon'orange'ghostwhite'lemon
menuGhostedColor = ShiftRGB(MenuColor, -contrast \ 2)
messageTextColor = textColor
messageColor = gadgetColor
' Chessboard colors
graph.boardColor = colors.lightYellow'lemonChiffon
graph.validDestinationColor = colors.yellowGreen
InitGFX
DisplayArbitratorMessage("")

dim shared as EventHandle ptr event
event = new EventHandle
dim as gadget ptr boardClickSensor = AddClickMapGadget(event, 0 + squareWidth \ 2, 18 + squareWidth \ 2, IMAGES_PATH & "clickmap.bmp")
GadgetOn(boardClickSensor)
dim as integer field_ID
dim as chessfield Board(1 to 64)

for k as integer = 0 to 7 
  Board(1 + k * 8).fieldname = "a" & str(k + 1) 
  Board(2 + k * 8).fieldname = "b" & str(k + 1) 
  Board(3 + k * 8).fieldname = "c" & str(k + 1) 
  Board(4 + k * 8).fieldname = "d" & str(k + 1) 
  Board(5 + k * 8).fieldname = "e" & str(k + 1) 
  Board(6 + k * 8).fieldname = "f" & str(k + 1) 
  Board(7 + k * 8).fieldname = "g" & str(k + 1) 
  Board(8 + k * 8).fieldname = "h" & str(k + 1) 
next k

dim shared appLog as integer
appLog = freefile
open LOG_NAME for output as #appLog
#define LogLn(x) print #appLog, x : fflush(cast(FILE ptr, fileattr(appLog, fbFileAttrHandle)))
LogLn("** " & PROGRAM_INFO)

dim shared as boolean autoPlay = true
dim shared as integer autoColor = chess.cBlack
dim shared engines() as TEngineInfo
LoadEnginesData(ENGINES_CONFIG)
dim engineIndex as integer = 0
dim as integer itemQuit, itemMove, itemAuto, itemNew, itemFlip, itemKnight, itemBishop, itemRook, itemQueen, itemEngines()

Menu(event, 1, 0, 1, PROGRAM_NAME)
itemQuit = Menu(event, 1, 1, 1, sQuit)
Menu(event, 2, 0, 1, sMoves)
itemMove = Menu(event, 2, 1, 1, sMove)
itemAuto = Menu(event, 2, 2, iif(autoplay, 3, 1), sAutoplay)
if ubound(engines) - lbound(engines) >= 0 then
  Menu(event, 2, 3, 4, "")
  redim itemEngines(lbound(engines) to ubound(engines))
  for i as integer = lbound(engines) to ubound(engines)
    itemEngines(i) = Menu(event, 2, i + 4, iif(i = engineIndex, 3, 1) - iif(engines(i).eiExists, 0, 1), engines(i).eiName)
  next i
end if
Menu(event, 3, 0, 1, sBoard)
itemNew = Menu(event, 3, 1, 1, sNew)
itemFlip = Menu(event, 3, 2, 1, sFlip)
Menu(event, 4, 0, 1, sPromotion)
itemKnight = Menu(event, 4, 1, 1, sKnight)
itemBishop = Menu(event, 4, 2, 1, sBishop)
itemRook = Menu(event, 4, 3, 1, sRook)
itemQueen = Menu(event, 4, 4, 3, sQueen)
MenuOn(event)

dim startingPosition as string = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1"
if (len(command(1)) > 0) andalso IsFEN(command(1)) then startingPosition = command(1)
graph.upsideDown = false
dim shared chessboard as graph.TGraphicalChessboard
chessboard.Create(0 + squareWidth \ 2, 18 + squareWidth \ 2)
chessboard.LoadPosition(mid(startingPosition, 1, instr(startingPosition, " ") - 1))
chessboard.RedrawPieces
if listener <> 0 then mutexlock(listener->sync)
screenlock
DrawCoord
chessboard.DrawToScreen
screenunlock
if listener <> 0 then mutexunlock(listener->sync)

dim shared game as chess.tChessGame
dim shared as string move, rookMove, enPassantCaptureSquare
dim shared as byte promotion
dim shared state as tGameState = gsCheckmate
dim shared check as boolean = false
dim shared history as stringlist
dim shared oneMove as boolean = false
dim shared userName as string
userName = environ("USERNAME")
if userName = "" then userName = environ("USER")
if userName = "" then userName = "?"
dim shared as string engineName, authorName
dim shared regExprCompiled as boolean
regExprCompiled = uci.CompileRegExpr
dim shared as string sequence
sequence = ""
dim shared as jsonItem ecoDatabase
ecoDatabase = jsonItem(trim(LoadFileAsString(OPENING_DATABASE), any chr(13, 10)))
dim shared as boolean searchOpeningName
searchOpeningName = (ecoDatabase.DataType = jsonObject)
game.Initialize(startingPosition)
OnNewPosition
dim engineConnected as boolean = engines(engineIndex).eiExists andalso ConnectEngine(engineIndex)
OnTryConnect(engineConnected)
dim shared waiting as boolean = false
InitSound(SOUND_ILLEGAL)

do
  DebugLn("")
  event->xSleep(1)
  DebugLn("")
  if engineConnected _
  andalso ComputerToMove then
    if not waiting then
      oneMove = false
      SendCommandLog("position fen " & game.FEN)
      SendCommandLog("go movetime " & str(moveTime))
      waiting = true
    end if
  end if
  
  if event->GADGETMESSAGE then
    DebugLn("GADGETMESSAGE")
    select case event->GADGETMESSAGE
      case boardClickSensor
        if waiting then
          continue do
        end if
        if state = gsInProgress then
          field_ID = getClickMapColor(boardClickSensor)
          move += iif(graph.upsideDown, ReplaceFieldName(Board(field_ID).fieldname), Board(field_ID).fieldname)
          if len(move) = 2 then
            if game.IsLegalStartingSquare(move) then
              chessboard.SaveImage
              var dest = game.LegalDestinations(move)
              for i as integer = 0 to len(dest) \ 2 - 1
                chessboard.Highlight(mid(dest, 2 * i + 1, 2))
              next
              chessboard.DrawToScreen
            else
              PlaySoundIllegal
              move = ""
            end if  
          end if
          if len(move) = 4 then
            chessboard.RestoreImage
            chessboard.DrawToScreen
            if game.IsLegal(move) then
              OnLegalMove(true) ' true => user move
              DisplayMove
              OnNewPosition
            else
              PlaySoundIllegal
              move = ""
            end if
          end if
        end if
    end select
  end if
  
  if event->MENUMESSAGE then
    DebugLn("MENUMESSAGE")
    select case event->MENUMESSAGE
      case itemQuit
        event->EXITEVENT = 1
      case itemMove
        if autoPlay then
          autoColor = chess.cBlack * autoColor
        else
          oneMove = true
        end if
      case itemAuto
        if GetChecked(event, 2, 2) then
          Menu(event, 2, 2, 1, sAutoplay)
        else
          Menu(event, 2, 2, 3, sAutoplay)
        end if
        autoPlay = GetChecked(event, 2, 2)
      case itemNew
        chessboard.ClearBoard
        chessboard.LoadPosition
        chessboard.RedrawPieces
        chessboard.DrawToScreen
        game.Initialize
        history.ClearList
        sequence = ""
        searchOpeningName = (ecoDatabase.DataType = jsonObject)
        OnNewPosition
      case itemFlip
        if len(move) = 2 then continue do
        graph.upsideDown = not graph.upsideDown
        chessboard.ClearBoard
        chessboard.FlipBoard
        chessboard.RedrawPieces
        mutexlock(listener->sync)
        screenlock
        RepaintBackground
        DrawCoord
        chessboard.DrawToScreen
        screenunlock
        mutexunlock(listener->sync)
      case itemKnight
        Menu(event, 4, 1, 3, sKnight)
        Menu(event, 4, 2, 1, sBishop)
        Menu(event, 4, 3, 1, sRook)
        Menu(event, 4, 4, 1, sQueen)
      case itemBishop
        Menu(event, 4, 1, 1, sKnight)
        Menu(event, 4, 2, 3, sBishop)
        Menu(event, 4, 3, 1, sRook)
        Menu(event, 4, 4, 1, sQueen)
      case itemRook
        Menu(event, 4, 1, 1, sKnight)
        Menu(event, 4, 2, 1, sBishop)
        Menu(event, 4, 3, 3, sRook)
        Menu(event, 4, 4, 1, sQueen)
      case itemQueen
        Menu(event, 4, 1, 1, sKnight)
        Menu(event, 4, 2, 1, sBishop)
        Menu(event, 4, 3, 1, sRook)
        Menu(event, 4, 4, 3, sQueen)
      case else
        for i as integer = lbound(itemEngines) to ubound(itemEngines)
          if event->MENUMESSAGE = itemEngines(i) then
            engineIndex = i
            for j as integer = lbound(itemEngines) to ubound(itemEngines)
              Menu(event, 2, j + 4, iif(j = engineIndex, 3, 1) + iif(engines(j).eiExists, 0, -1), engines(j).eiName)
            next j
            if listener <> 0 then
              listener->doReadOutput = false
              SendCommandLog("quit")
            end if
            engineConnected = ConnectEngine(engineIndex)
            OnTryConnect(engineConnected)
          end if
        next i
    end select
  end if
loop until event->EXITEVENT

if engineConnected then
  listener->doReadOutput = false
  SendCommandLog("quit")
end if
delete event
chessboard.Free
close #appLog
if listener <> 0 then
  listener->quit = true
  threadwait(listener->handle)
  mutexdestroy(listener->sync)
  delete listener
  listener = 0
end if
Disconnect
#ifdef DEBUG
close console
#endif
SaveSettings(moveTime, squareWidth)

sub DisplayArbitratorMessage(byval s as string = "")
  line (0, 18 + BOARD_WIDTH + squareWidth)-(BOARD_WIDTH + squareWidth - 1, 18 + BOARD_WIDTH + 18 + squareWidth - 1), messageColor, BF
  draw string (8, 18 + BOARD_WIDTH + squareWidth + (18 - textHeight) \ 2), s, messageTextColor
end sub

function IsPromotionMove(byref g as chess.tChessGame, byval m as string) as boolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cPawn) andalso ((m[3] - 48 = 1) or (m[3] - 48 = 8))
end function

function IsCastlingMove(byref g as chess.tChessGame, byval m as string) as boolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cKing) andalso (abs(m[0] - m[2]) = 2)
end function

function IsEnPassantMove(byref g as chess.tChessGame, byval m as string) as boolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cPawn) andalso ((abs(g.board(m[2] - 96, m[3] - 48)) = chess.cNothing) and (m[0] <> m[2]))
end function

function ReplaceFieldName(byval s as string) as string
  dim as ubyte asc1 = s[0] - 96, asc2 = s[1] - 48
  asc1 = 9 - asc1
  asc2 = 9 - asc2
  return chr(asc1 + 96, asc2 + 48)
end function

function ComparePiecePlacement(byval FENRecord1 as string, byval FENRecord2 as string) as boolean
  return mid(FENRecord1, 1, instr(FENRecord1, " ")) = mid(FENRecord2, 1, instr(FENRecord2, " "))
end function

function ComputerToMove as boolean
  return cbool(state = gsInProgress) andalso ((autoPlay andalso cbool(autoColor = game.activeColor)) or oneMove)
end function

function LoadFileAsString(byval filename as string) as string
  dim s as string
  dim vFile as integer = FreeFile()
  if open(filename for input access read as #vFile) = 0 then
    close #vFile
    if open(filename for binary access read as #vFile) = 0 then
      s = space(lof(vFile))
      get #vFile,, s
      close #vFile
    end if 
  end if
  return s
end function

sub LoadEnginesData(aFileName as const string)
  dim as string jsontext = trim(LoadFileAsString(aFileName), any chr(13, 10))
  dim as jsonItem item1, item = jsonItem(jsontext)
  dim ei as TEngineInfo
  dim fileName as string
  if item.Datatype = jsonArray then
    redim engines(item.Count - 1)
    for i as integer = 0 to item.Count - 1
      item1 = jsonItem(item[i])
      if item[i].DataType = jsonObject then
        with ei
          .eiCommand = item1["command"].Value
          .eiName = item1["name"].Value
          .eiWorkingDirectory = exepath & item1["workingDirectory"].Value
          filename = ei.eiCommand
          if mid(fileName, 1, 2) = "./" then fileName = mid(fileName, 3)
          fileName = ei.eiWorkingDirectory & fileName
          .eiExists = fileexists(fileName)
          if not .eiExists then LogLn("** Cannot find " & fileName)
        end with
        engines(i) = ei
      end if
    next i
  end if
end sub

sub ProcedureThread(byval param as any ptr)
  DebugLn("->")
  dim engineOutput as string
  with *cast(TListener ptr, param)
    do
      mutexlock(.sync)
      if .doReadOutput then
        engineOutput = ReadEngineOutput
      else
        engineOutput = ""
      end if
      if engineOutput <> "" then
        .procedure(engineOutput)
      end if
      mutexunlock(.sync)
      sleep(50, 1)
    loop until .quit
  end with
  DebugLn("<-")
end sub

function FormatEngineOutput(byref aStr as const string) as string
#ifdef __FB_WIN32__
const EOL = chr(13, 10)
#endif
#ifdef __FB_LINUX__
const EOL = chr(10)
#endif
  dim s as string = trim(aStr, any chr(13, 10))
  dim as integer i = instrrev(s, any chr(13, 10))
  do while i > 0
    s = trim(mid(s, 1, i), any chr(13, 10)) & EOL & "<< " & mid(s, i + 1)
    i = instrrev(s, any chr(13, 10), i - 2)
  loop
  return "<< " & s
end function

sub OnOutput(aOutput as const string)
  DebugLn("->")
  LogLn(FormatEngineOutput(aOutput))
  if instrrev(aOutput, "uciok") > 0 then
    OnUciOk(aOutput)
  elseif instrrev(aOutput, "bestmove") > 0 then
    OnBestMove(aOutput)
  end if
  DebugLn("<-")
end sub

sub OnUciOk(aOutput as const string)
  DebugLn("->")
  if uci.ParseUciOk(aOutput, engineName, authorName) then
    LogLn("** Engine name: [" & engineName & "]")
    LogLn("** Engine author: [" & authorName & "]")
  else
    engineName = "?"
    authorName = "?"
  end if
  windowtitle(PROGRAM_NAME & " " & PROGRAM_VERSION & " [" & engineName & "]")
  DebugLn("<-")
end sub

sub OnBestMove(aOutput as const string)
  DebugLn("->")
  dim as integer iStart, iEnd
  iStart = instrrev(aOutput, "bestmove") + len("bestmove ")
  iEnd = instr(iStart, aOutput, any chr(10, 13, 32))
  move = mid(aOutput, iStart, iEnd - iStart)
  LogLn("** Engine move: " & move)
  if cbool(len(move) >= 4) andalso game.IsLegal(move) then
    OnLegalMove(false) ' false => computer move
    DisplayMove
    OnNewPosition
  else
    LogLn("** Engine played an illegal move")
    autoPlay = false
    if GetChecked(event, 2, 2) then Menu(event, 2, 2, 1, sAutoplay)
    DisplayArbitratorMessage(sEngineError & iif(game.activeColor = chess.cWhite, sWhiteToMove, sBlackToMove))
  end if
  waiting = false
  DebugLn("<-")
end sub

function ConnectEngine(aIndex as const integer) as boolean
  DebugLn("->")
  dim res as boolean = cbool(aIndex >= lbound(engines)) _
  andalso cbool(aIndex <= ubound(engines)) _
  andalso cbool(chdir(engines(aIndex).eiWorkingDirectory) = 0) _
  andalso Connect(engines(aIndex).eiCommand) _
  andalso regExprCompiled
  DebugLn("<-")
  return res
end function

function DetectRepetition as boolean
  dim as integer vCount = 0
  for i as integer = 0 to history.Count - 2
    if ComparePiecePlacement(history.Strings(i), history.Strings(history.Count - 1)) then vCount += 1
  next
  return cbool(vCount >= 2)
end function

sub SendCommandLog(byref aCommand as const string)
  LogLn(">> " & aCommand)
  SendCommand(aCommand)
end sub

sub OnNewPosition
  DebugLn("->")
  history.Add(game.FEN)
  game.GenerateLegalMoves
  state = game.GameState
  if cbool(state = gsInProgress) andalso DetectRepetition then
    state = gsDraw
  end if
  if state = gsInProgress then
    check = game.Check
    DisplayArbitratorMessage(iif(check, sCheck, "") & iif(game.activeColor = chess.cWhite, sWhiteToMove, sBlackToMove) & IIF(ComputerToMove(), "..", ""))
  else
    DisplayArbitratorMessage(sState(state))
  end if
  if searchOpeningName andalso (len(sequence) > 0) then
    if ecoDatabase.ContainsKey(sequence) then
      LogLn("** Opening name: " & mid(ecoDatabase[sequence].Value, 5))
    else
      searchOpeningName = false
    end if
  end if
  DebugLn("<-")
end sub

sub OnLegalMove(byref aUserMove as const boolean)
  DebugLn("->")
  dim piece as integer = chess.cNothing
  dim symbols as string = "  nbrq"
  if IsPromotionMove(game, move) then
    if aUserMove then
      if     GetChecked(event, 4, 1) then
        piece = chess.cKnight
      elseif GetChecked(event, 4, 2) then
        piece = chess.cBishop
      elseif GetChecked(event, 4, 3) then
        piece = chess.cRook
      elseif GetChecked(event, 4, 4) then
        piece = chess.cQueen
      end if
    else
      if len(move) = 5 then
        var s = mid(move, 5, 1)
        var i = instr("nbrq", lcase(s))
        select case i
          case 0 : piece = chess.cQueen : LogLn("** Unexpected symbol: " & s)
          case 1 : piece = chess.cKnight
          case 2 : piece = chess.cBishop
          case 3 : piece = chess.cRook
          case 4 : piece = chess.cQueen
        end select
      else
        LogLn("** Missing promotion symbol")
        piece = chess.cQueen
      end if
    end if
    game.DoMove(move, piece)
    promotion = symbols[piece]
  else
    if IsCastlingMove(game, move) then
      select case move
        case "e1g1" : rookMove = "h1f1"
        case "e1c1" : rookMove = "a1d1"
        case "e8g8" : rookMove = "h8f8"
        case "e8c8" : rookMove = "a8d8"
      end select
    elseif IsEnPassantMove(game, move) then
      enPassantCaptureSquare = chr(move[2], move[1])
    end if
    game.DoMove(move)
  end if
  LogLn("** " & iif(aUserMove, "User", "Engine") & " [" & iif(aUserMove, userName, engineName) & "] plays " & move)
  if searchOpeningName then
    if len(sequence) < 100 then
      sequence &= move
    else
      sequence = ""
      searchOpeningName = false
    end if
  end if
  DebugLn("<-")
end sub

sub OnTryConnect(byref aSuccess as const boolean)
  DebugLn("->")
  if aSuccess then
    listener->doReadOutput = true
    sleep(1000, 1)
    SendCommandLog("uci")
  else
    autoPlay = false
    Menu(event, 2, 2, 2, sAutoplay)
    LogLn("** Connection failed")
  end if
  DebugLn("<-")
end sub

sub DisplayMove
  DebugLn("->")
  if promotion <> 0 then
    chessboard.Promote(left(move, 2), promotion)
    chessboard.DrawToScreen
    sleep(10, 1)
  end if
  if enPassantCaptureSquare <> "" then
    chessboard.HidePiece(enPassantCaptureSquare)
    chessboard.DrawToScreen
    sleep(10, 1)
  endif
  if len(move) >= 4 then
    chessboard.MovePiece(move)
    while chessboard.Animate
      chessboard.DrawToScreen
      sleep(10, 1)
    wend
  end if
  if rookMove <> "" then
    chessboard.MovePiece(rookMove)
    while chessboard.Animate
      chessboard.DrawToScreen
      sleep(10, 1)
    wend
  end if
  promotion = 0
  enPassantCaptureSquare = ""
  move = ""
  rookMove = ""
  DebugLn("<-")
end sub

function StrToIntDef(byref aStr as const string, byref default as const integer) as integer
  if (aStr = "") or (len(aStr) > 9) then return default
  for i as integer = 0 to len(aStr) - 1
    if (aStr[i] < asc("0")) or (aStr[i] > asc("9")) then return default
  next
  return valint(aStr)
end function

sub RepaintBackground
  line (0, 18)-step(9 * squareWidth, 9 * squareWidth), backgroundColor, bf
end sub

sub DrawCoord
  dim char as string
  for y as integer = 0 to 7
    if graph.upsideDown then
      char = chr(asc("1") + y)
    else
      char = chr(asc("8") - y)
    end if
    draw string (((squareWidth \ 2) - 8) \ 2, 18 + squareWidth \ 2 + y * squareWidth + (squareWidth - textHeight) \ 2), char
  next y 
  for x as integer = 0 to 7
    if graph.upsideDown then
      char = chr(asc("H") - x)
    else
      char = chr(asc("A") + x)
    end if
    draw string (squareWidth \ 2 + x * squareWidth + (squareWidth - 8) \ 2, 18 + BOARD_WIDTH + squareWidth \ 2 + (squareWidth \ 2 - textHeight) \ 2), char
  next x
end sub

sub LoadSettings(byref aMoveTime as integer, byref aSquareWidth as integer)
  const defaultMoveTime = 1000
  const defaultSquareWidth = 40
  dim iniFile as iniobj = iniobj(INI_NAME)
  dim s as string
  s = iniFile.ReadString("settings", "movetime", str(defaultMoveTime))
  aMoveTime = StrToIntDef(s, defaultMoveTime)
  s = iniFile.ReadString("settings", "squarewidth", str(defaultSquareWidth))
  aSquareWidth = StrToIntDef(s, defaultSquareWidth)
end sub

sub SaveSettings(byref aMoveTime as const integer, byref aSquareWidth as const integer)
  dim iniFile as iniobj = iniobj(INI_NAME)
  iniFile.WriteString("settings", "movetime", str(aMoveTime))
  iniFile.WriteString("settings", "squarewidth", str(aSquareWidth))
  iniFile.UpdateFile
end sub
