
#define DEFAULT_PIECE_PLACEMENT "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR"
#define PIECES_COUNT 32
#define BOARD_WIDTH (8 * squareWidth)
#define STEP_LENGTH (squareWidth \ 8)
#define IMAGES_PATH exepath & "/resources/pictures/" & squareWidth & "/"

dim shared as integer squareWidth = 40

namespace graph

dim shared as boolean upsideDown = false
dim shared as uinteger boardColor = colors.lemonChiffon
dim shared as uinteger validDestinationColor = colors.yellowGreen

function XToScreen(x as integer) as integer
  return iif(upsideDown, squareWidth * (8 - x), squareWidth * (x - 1))
end function

function YToScreen(y as integer) as integer
  return iif(upsideDown, squareWidth * (y - 1), squareWidth * (8 - y))
end function

type TChessImage
  as any ptr fBackground, fForeground
  as integer fLeft, fTop
  declare sub Create
  declare sub Reload(byval aFilename as string)
  declare sub Free
  declare sub Redraw(byval aBuffer as any ptr, byval aLeft as integer, byval aTop as integer)
  declare sub Hide(byval aBuffer as any ptr)
end type

type TMovingChessImage
  as TChessImage img
  as integer fX, fY, fStepX, fStepY, fTargetX, fTargetY
  declare sub Create
  declare sub Reload(byval aWhat as string)
  declare sub Free
  declare sub DrawImage overload(byval aBuffer as any ptr, byval aX as integer, byval aY as integer)
  declare sub DrawImage overload(byval aBuffer as any ptr, byval aSquare as string)
  declare sub PrepareAnimation(byval aX as integer, byval aY as integer)
  declare function MoveImage(byval aBuffer as any ptr) as boolean
end type

type TGraphicalChessboard
  dim fPieceImage(PIECES_COUNT - 1) as TMovingChessImage
  dim fPieceType(PIECES_COUNT - 1) as string * 2
  dim fPieceLocation(PIECES_COUNT - 1) as string * 2
  dim fPieceCount as integer = PIECES_COUNT
  dim fImage as any ptr
  dim fBackup as any ptr
  dim fValid as any ptr
  dim fImageIndex as integer
  dim as integer fLeft = 0, fTop = 0
  declare sub Create(byval aLeft as integer, byval aTop as integer)
  declare sub Free
  declare sub LoadPosition(byval aPlacement as string = DEFAULT_PIECE_PLACEMENT)
  declare sub RedrawPieces
  declare sub ClearBoard
  declare sub MovePiece overload(byval aX1 as integer, byval aY1 as integer, byval aX2 as integer, byval aY2 as integer)
  declare sub MovePiece overload(byval move as string)
  declare function Animate as boolean
  declare function HidePiece overload(byval aX as integer, byval aY as integer) as integer
  declare function HidePiece overload(byval square as string) as integer
  declare sub Promote(byval aWhere as string, byval aNewPieceSymbol as byte)
  declare sub Highlight overload(byval aX as integer, byval aY as integer)
  declare sub Highlight overload(byval square as string)
  declare sub SaveImage
  declare sub RestoreImage
  declare sub DrawToScreen
  declare sub FlipBoard
end type

sub TChessImage.Create
  DebugLn("->")
  fBackground = imagecreate(squareWidth, squareWidth)
  fForeground = imagecreate(squareWidth, squareWidth)
  fLeft = -1
  fTop = -1
  DebugLn("<-")
end sub

sub TChessImage.Reload(byval aFilename as string)
  DebugLn("->")
  dim res as integer
  res = bload(IMAGES_PATH & aFilename, fForeground)
  if res <> 0 then
    DebugLn("res = " & str(res))
  end if
  DebugLn("<-")
end sub

sub TChessImage.Free
  DebugLn("->")
  imagedestroy(fBackground)
  imagedestroy(fForeground)
  DebugLn("<-")
end sub

sub TChessImage.Redraw(byval aBuffer as any ptr, byval aLeft as integer, byval aTop as integer)
  DebugLn("->")
  if fLeft <> -1 then put aBuffer, (fLeft, fTop), fBackground, pset
  fLeft = aLeft
  fTop = aTop
  get aBuffer, (fLeft, fTop)-(fLeft + squareWidth - 1, fTop + squareWidth - 1), fBackground
  put aBuffer, (fLeft, fTop), fForeground, trans
  DebugLn("<-")
end sub

sub TChessImage.Hide(byval aBuffer as any ptr)
  DebugLn("->")
  if fLeft <> -1 then put aBuffer, (fLeft, fTop), fBackground, pset
  fLeft = -1
  fTop = -1
  DebugLn("<-")
end sub

sub TMovingChessImage.Create
  DebugLn("->")
  img.Create
  fX = -1
  fY = -1
  fStepX = 0
  fStepY = 0
  fTargetX = 0
  fTargetY = 0
  DebugLn("<-")
end sub

sub TMovingChessImage.Reload(byval aWhat as string)
  DebugLn("->")
  img.Reload(aWhat & ".bmp")
  DebugLn("<-")
end sub

sub TMovingChessImage.Free
  DebugLn("->")
  img.Free
  DebugLn("<-")
end sub

sub TMovingChessImage.DrawImage overload(byval aBuffer as any ptr, byval aX as integer, byval aY as integer)
  DebugLn("->")
  fX = aX
  fY = aY
  img.Redraw(aBuffer, XToScreen(fX), YToScreen(fY))
  DebugLn("<-")
end sub

sub TMovingChessImage.DrawImage overload(byval aBuffer as any ptr, byval aSquare as string)
  dim as integer x, y
  x = aSquare[0] - asc("a") + 1
  y = aSquare[1] - asc("1") + 1
  DrawImage(aBuffer, x, y)
end sub

sub TMovingChessImage.PrepareAnimation(byval aX as integer, byval aY as integer)
  DebugLn("->")
  dim as integer dx = aX - fX
  dim as integer dy = aY - fY
  ' Calcul des vecteurs pour le déplacement de l'image.
  fStepX = sgn(dx) ' -1, 0 ou 1
  fStepY = sgn(dy) ' -1, 0 ou 1
  dx = abs(dx)
  dy = abs(dy)
  ' Ajustement pour les mouvements du cavalier.
  if dx = 2 * dy then fStepX *= 2
  if dy = 2 * dx then fStepY *= 2
  ' Inversion des valeurs si l'échiquier est à l'envers.
  if upsideDown then
    fStepX *= -1
    fStepY *= -1
  end if
  ' Multiplication par la longueur du pas.
  fStepX *= STEP_LENGTH
  fStepY *= STEP_LENGTH
  ' S'il y a plus de trois cases à parcourir, doubler la longueur du pas.
  if (dx > 3) or (dy > 3) then
    fStepX *= 2
    fStepY *= 2
  end if
  ' Emplacement final de l'image sur l'image de l'échiquier.
  fTargetX = XToScreen(fX)
  fTargetY = YToScreen(fY)
  ' Emplacement final de la pièce sur l'échiquier.
  fX = aX
  fY = aY
  DebugLn("<-")
end sub

function TMovingChessImage.MoveImage(byval aBuffer as any ptr) as boolean
  if (fTargetX = XToScreen(fX)) and (fTargetY = YToScreen(fY)) then
    return false
  else
    fTargetX += fStepX
    fTargetY -= fStepY
    img.Redraw(aBuffer, fTargetX, fTargetY)
    return true
  end if
end function

sub TGraphicalChessboard.Create(byval aLeft as integer, byval aTop as integer)
  dim res as integer
  dim darkSquare as any ptr
  dim board as any ptr = imagecreate(BOARD_WIDTH, BOARD_WIDTH)
  
  fLeft = aLeft
  fTop = aTop
  
  fImage = imagecreate(BOARD_WIDTH, BOARD_WIDTH)
  line fImage, (0, 0)-(BOARD_WIDTH - 1, BOARD_WIDTH - 1), boardColor, bf
  
  if squareWidth = 40 then
  ' Chargement de l'échiquier de 320.
    res = bload(IMAGES_PATH & "board.bmp", board)
    if res <> 0 then
      DebugLn("res = " & str(res))
    end if
  else
  ' Création de l'échiquier de 368.
    darkSquare = imagecreate(squareWidth, squareWidth)
    for x as integer = 0 to squareWidth - 1
      for y as integer = 0 to squareWidth - 1
        if (x + y) mod 5 = 2 then
          pset darkSquare, (x, y), colors.black
        end if
      next y
    next x
    for y as integer = 0 to 7
      for x as integer = 0 to 7
        if (x + y) mod 2 = 1 then
          put board, (squareWidth * x, squareWidth * y), darkSquare, trans
        end if
      next x
    next y
    imagedestroy darkSquare
  end if
  put fImage, (0, 0), board, trans
  imagedestroy board
  for i as integer = 0 to fPieceCount - 1
    fPieceImage(i).Create
  next i
  fBackup = imagecreate(BOARD_WIDTH, BOARD_WIDTH)
  fValid = imagecreate(squareWidth, squareWidth) 
  line fValid, (0, 0)-(squareWidth - 1, squareWidth - 1), boardColor, bf
  line fValid, (4, 4)-(squareWidth - 5, squareWidth - 5), validDestinationColor, bf
end sub

sub TGraphicalChessboard.Free
  for i as integer = 0 to fPieceCount - 1
    fPieceImage(i).Free
  next i
  imagedestroy fImage
  imagedestroy fBackup
  imagedestroy fValid
end sub

sub TGraphicalChessboard.LoadPosition(byval aPlacement as string)
  dim as byte bs = asc("/"), b1 = asc("1"), b8 = asc("8"), ba = asc("a")
  dim as integer x = 1, y = 8, j = 0
  dim n as integer
  for i as integer = 0 to len(aPlacement) - 1
    if aPlacement[i] = bs then
      x = 1
      y -= 1
    else
      if (aPlacement[i] >= b1) and (aPlacement[i] <= b8) then
        n = aPlacement[i] - b1 + 1
        do while n >= 1
          x += 1
          n -= 1
        loop
      else
        fPieceType(j) = iif(aPlacement[i] < ba, "w", "b") & lcase(chr(aPlacement[i]))
        fPieceLocation(j) = chr(x + ba - 1, y + b1 - 1)
        fPieceImage(j).Reload(fPieceType(j))
        x += 1
        j += 1
      end if
    end if
  next i
  fPieceCount = j
end sub

sub TGraphicalChessboard.RedrawPieces
  for i as integer = 0 to fPieceCount - 1
    fPieceImage(i).DrawImage(fImage, fPieceLocation(i))
  next i
end sub

sub TGraphicalChessboard.ClearBoard
  for i as integer = 0 to fPieceCount - 1
    fPieceImage(i).img.Hide(fImage)
  next i
end sub

sub TGraphicalChessboard.MovePiece overload( _
  byval aX1 as integer, _
  byval aY1 as integer, _
  byval aX2 as integer, _
  byval aY2 as integer _
)
  DebugLn("->")
  dim i as integer = 0
  dim found as boolean = false
  while (i <= PIECES_COUNT - 1) and (not found)
    if (fPieceImage(i).fX = aX2) and (fPieceImage(i).fY = aY2) then
      found = true
      fPieceImage(i).img.Hide(fImage)
      fPieceImage(i).fX = -1
      fPieceImage(i).fY = -1
      fPieceLocation(i) = "??"
    else
      i += 1
    end if
  wend
  i = 0
  found = false
  while (i <= PIECES_COUNT - 1) and (not found)
    if (fPieceImage(i).fX = aX1) and (fPieceImage(i).fY = aY1) then
      found = true
    else
      i += 1
    end if
  wend
  if found then
    fPieceImage(i).PrepareAnimation(aX2, aY2)
    fPieceLocation(i) = chr(aX2 + asc("a") - 1, aY2 + asc("1") - 1)
    fImageIndex = i
  end if
  DebugLn("<-")
end sub

sub TGraphicalChessboard.MovePiece overload(byval move as string)
  MovePiece( _
    move[0] - asc("a") + 1, _
    move[1] - asc("1") + 1, _
    move[2] - asc("a") + 1, _
    move[3] - asc("1") + 1 _
 )
end sub

function TGraphicalChessboard.Animate as boolean
  return fPieceImage(fImageIndex).MoveImage(fImage)
end function

function TGraphicalChessboard.HidePiece overload(byval aX as integer, byval aY as integer) as integer
  dim as integer i, bx, by, res = 1
  i = 0
  while (i <= PIECES_COUNT - 1) and (res = 1)
    if (fPieceImage(i).fX = aX) and (fPieceImage(i).fY = aY) then
      res = 0
    else
      i += 1
    end if
  wend
  if res = 0 then
    fPieceImage(i).img.Hide(fImage)
    fPieceImage(i).fX = -1
    fPieceImage(i).fY = -1
  end if
  return res
end function

function TGraphicalChessboard.HidePiece overload(byval square as string) as integer
  return HidePiece( _
    square[0] - asc("a") + 1, _
    square[1] - asc("1") + 1 _
 )
end function

sub TGraphicalChessboard.Promote(byval aWhere as string, byval aNewPieceSymbol as byte)
  dim i as integer = 0
  do while (i <= fPieceCount - 1) andalso (fPieceLocation(i) <> aWhere)
    i += 1  
  loop
  if i <= fPieceCount - 1 then
    fPieceImage(i).img.Hide(fImage)
    fPieceType(i)[1] = aNewPieceSymbol
    fPieceImage(i).img.Reload(fPieceType(i) & ".bmp")
    fPieceImage(i).DrawImage(fImage, fPieceLocation(i))
  end if
end sub

sub TGraphicalChessboard.Highlight overload(_
    byval aX as integer, _
    byval aY as integer _
 )
  put fImage, (XToScreen(aX), YToScreen(aY)), fValid, and
end sub  

sub TGraphicalChessboard.Highlight overload(byval square as string)
  Highlight(square[0] - asc("a") + 1, square[1] - asc("1") + 1)
end sub
 
sub TGraphicalChessboard.SaveImage
  get fImage, (0, 0)-(BOARD_WIDTH - 1, BOARD_WIDTH - 1), fBackup
end sub

sub TGraphicalChessboard.RestoreImage
  put fImage, (0, 0), fBackup, pset
end sub

sub TGraphicalChessboard.DrawToScreen
  DebugLn("->")
  put (fLeft, fTop), fImage, pset
  DebugLn("<-")
end sub

sub TGraphicalChessboard.FlipBoard
  for i as integer = 0 to fPieceCount - 1
    fPieceImage(i).img.fLeft = XToScreen(fPieceImage(i).fX)
    fPieceImage(i).img.fTop = YToScreen(fPieceImage(i).fY)
  next i
end sub

end namespace
