
#include once "pcre.bi"

namespace uci

const MOVE_PATTERN = "bestmove ([a-h])([1-8])([a-h])([1-8])([nbrq]?)"
const MOVE_PATTERN_CAPTURES = 5
const NIL as byte = 0
const UCIOK_PATTERN = "id name (.+?)[\r\n]+id author (.+?)[\r\n]+"
const UCIOK_PATTERN_CAPTURES = 2

dim shared as pcre ptr re, re2

declare function CompileRegExpr as boolean
declare function ParseBestMove(buffer as const string, aBestMove() as byte) as boolean
declare function ParseUciOk(buffer as const string, byref aName as string,  byref aAuthor as string) as boolean

function CompileRegExpr as boolean
  dim as zstring ptr error_
  dim as long error_offset
  dim vResult as boolean = true
  re = pcre_compile(MOVE_PATTERN, 0, @error_, @error_offset, NULL)
  if re = NULL then
    'print("Compilation failed at offset " & str(error_offset) & ": " & *error_)
    'end
    vResult = false
  end if
  re2 = pcre_compile(UCIOK_PATTERN, 0, @error_, @error_offset, NULL)
  if re2 = NULL then
    'print("Compilation failed at offset " & str(error_offset) & ": " & *error_)
    'end
    vResult = false
  end if
  return vResult
end function

function ParseBestMove(buffer as const string, aBestMove() as byte) as boolean
  const VECTOR_COUNT = 3 * (MOVE_PATTERN_CAPTURES + 1)
  dim as integer rc
  dim as long ovector(VECTOR_COUNT - 1) 
  dim as boolean vResult = false
  var start_offset = 0
  do
    rc = pcre_exec(re, NULL, buffer, len(buffer), start_offset, 0, @ovector(0), VECTOR_COUNT)
    if rc < 0 then
      exit do
    end if
    if rc = 0 then
      rc = VECTOR_COUNT \ 3
    end if
    vResult = rc = MOVE_PATTERN_CAPTURES + 1
    for i as integer = 0 to rc - 1
      var aux = mid(buffer, ovector(2 * i) + 1, ovector(2 * i + 1) - ovector(2 * i))
      if len(aux) = 1 then
        aBestMove(i) = aux[0]
      else
        aBestMove(i) = NIL
      end if
    next i
    start_offset = ovector(2 * (rc - 1) + 1)
  loop
  return vResult
end function

function ParseUciOk(buffer as const string, byref aName as string,  byref aAuthor as string) as boolean
  const VECTOR_COUNT = 3 * (UCIOK_PATTERN_CAPTURES + 1)
  dim as integer rc
  dim as long ovector(VECTOR_COUNT - 1) 
  dim as boolean vResult = false
  var start_offset = 0
  do
    rc = pcre_exec(re2, NULL, buffer, len(buffer), start_offset, 0, @ovector(0), VECTOR_COUNT)
    if rc < 0 then
      exit do
    end if
    if rc = 0 then
      rc = VECTOR_COUNT \ 3
    end if
    vResult = rc = UCIOK_PATTERN_CAPTURES + 1
    if vResult then
      var i = 1
      aName = mid(buffer, ovector(2 * i) + 1, ovector(2 * i + 1) - ovector(2 * i))
      i = 2
      aAuthor = mid(buffer, ovector(2 * i) + 1, ovector(2 * i + 1) - ovector(2 * i))
    else
      aName = "?"
      aAuthor = "?"
    end if
    start_offset = ovector(2 * (rc - 1) + 1)
  loop
  return vResult
end function

end namespace
