
#include once "pcre.bi"

function IsMatch(byval aPattern as string, byval aSubject as string) as integer
  dim regex as pcre ptr
  dim errcode as zstring ptr 
  dim erroffset as long
  dim ovector as long
  regex = pcre_compile(aPattern, 0, @errcode, @erroffset, 0)
  if regex = 0 then
    return 0
  end if
  return pcre_exec(regex, 0, aSubject, len(aSubject), 0, 0, @ovector, 1) = 0
end function

sub Split(byval aString as string, byval aSeparator as string, aResult() as string)
  dim as string s1, s2
  dim as integer i, n = 0
  do
    i = instr(aString, aSeparator)
    s1 = ""
    s2 = ""
    if i <> 0 then
      s1 = mid(aString, 1, i - 1)
      s2 = mid(aString, i + 1)
    else
      s1 = aString
    endif
    redim preserve aResult(1 to 1 + n - ((len(s1) > 0) + (len(s2) > 0)))
    aResult(n + 1) = s1
    n = n + 1
    aString = s2
  loop until s2 = ""
  redim preserve aResult(1 to ubound(aResult) - 1)
end sub

function SquaresCount(byval s as string) as integer
  dim as integer i, result = 0
  for i = 0 to len(s) - 1
    if instr("BKNPQRbknpqr", chr(s[i])) then
      result += 1
    else
      result += s[i] - asc("0")
    end if
  next i
  return result
end function

function IsFEN(byval s as string) as integer
  
  const RE_WHITEKING = "K"
  const RE_BLACKKING = "k"
  const RE_PIECES = "^[1-8BKNPQRbknpqr]+$"
  const RE_ACTIVECOLOR = "^[wb]$"
  const RE_CASTLING = "^[KQkq]+$|^\-$"
  const RE_ENPASSANT = "^[a-h][36]$|^\-$"
  const RE_HALFMOVE = "^\d+$"
  const RE_FULLMOVE = "^[1-9]\d*$"
  
  redim as string a(), b()
  dim as integer i, result = (0 = 0)
  
  Split(s, " ", a())
  Split(a(1), "/", b())
  
  result = result and (ubound(a) = 6)
  result = result and (ubound(b) = 8)
  if result then
    result = result and IsMatch(RE_WHITEKING, a(1))
    result = result and IsMatch(RE_BLACKKING, a(1))
    for i = lbound(b) to ubound(b)
      result = result and IsMatch(RE_PIECES, b(i))
      result = result and (SquaresCount(b(i)) = 8)
    next i
    result = result and IsMatch(RE_ACTIVECOLOR, a(2))
    result = result and IsMatch(RE_CASTLING,    a(3))
    result = result and IsMatch(RE_ENPASSANT,   a(4))
    result = result and IsMatch(RE_HALFMOVE,    a(5))
    result = result and IsMatch(RE_FULLMOVE,    a(6))
  end if
  
  return result
end function
